import { ApolloClient, InMemoryCache } from "@apollo/client";
import { prepareRoutes } from "@curi/router";
import { omit } from 'ramda';
import React from 'react';
import { CreateProjectInput, Pipeline, Project } from './generated/graphql';
import { ProjectDetail, ProjectEditor, PROJECT } from "./projects";
import { COMMIT_LIST_QUERY } from './commons/graphql/queries';
import { CommitList } from './commits/commit-list';
import { PipelineTaskDetail } from './pipeline-tasks/pipeline-task-detail';

export default prepareRoutes([
  {
    name: "dashboard",
    path: "",
    respond() {
      return { body: () => <div>DashBoard</div> };
    },
  }, // dashboard
  {
    name: "create-project",
    path: "projects/create",
    respond({ resolved }) {
      const input: CreateProjectInput = {
        name: "",
        comment: "",
        webHookSecret: "",
        sshUrl: "",
        webUrl: "",
      };
      return { body: () => <ProjectEditor project={input} /> };
    },
  }, // create-project
  {
    name: "edit-project",
    path: "projects/:projectId/edit",
    async resolve(
      matched,
      { client }: { client: ApolloClient<InMemoryCache> }
    ) {
      const { data } = await client.query<{ project: Project }>({
        query: PROJECT,
        variables: { id: matched?.params.projectId },
      });
      return {
        body: () => (
          <ProjectEditor project={omit(["__typename"], data.project)} />
        ),
      };
    },
    respond({ resolved }) {
      return resolved;
    },
  }, // edit-project
  {
    name: "project-detail",
    path: "projects/:projectId",
    async resolve(
      matched,
      { client }: { client: ApolloClient<InMemoryCache> }
    ) {
      const { data } = await client.query<{ project: Project }>({
        query: PROJECT,
        variables: { id: matched?.params.projectId },
      });
      return {
        body: () => (
          <ProjectDetail project={omit(["__typename"], data.project)} />
        ),
      };
    },
    respond({ resolved }) {
      return resolved;
    },
    children: [
      {
        name: "pipeline-commits",
        path: "pipelines/:pipelineId/commits",
        async resolve(
          matched,
          { client }: { client: ApolloClient<InMemoryCache> }
        ) {
          const { data } = await client.query<{
            pipeline: Pipeline;
            project: Project;
          }>({
            query: COMMIT_LIST_QUERY,
            variables: {
              projectId: matched?.params.projectId,
              pipelineId: matched?.params.pipelineId,
            },
          });
          return {
            body: () => (
              <ProjectDetail project={omit(["__typename"], data.project)}>
                <CommitList pipeline={omit(["__typename"], data.pipeline)} />
              </ProjectDetail>
            ),
          };
        },
        respond({ resolved, error }) {
          return resolved || <div>Failed</div>;
        },
      },
      {
        name: "pipeline-task-detail",
        path: "pipelines/:pipelineId/tasks/:taskId",
        async resolve(
          matched,
          { client }: { client: ApolloClient<InMemoryCache> }
        ) {
          const { data } = await client.query<{
            pipeline: Pipeline;
            project: Project;
          }>({
            query: COMMIT_LIST_QUERY,
            variables: {
              projectId: matched?.params.projectId,
              pipelineId: matched?.params.pipelineId,
            },
          });
          return {
            body: () => (
              <ProjectDetail project={omit(["__typename"], data.project)}>
                <PipelineTaskDetail taskId={matched?.params.taskId} />
              </ProjectDetail>
            ),
          };
        },
        respond({ resolved, error }) {
          return resolved || <div>Failed</div>;
        },
      },
    ],
  }, // project-detail
]);
